#!/bin/bash
# modemlink - start device drivers and set /dev/modem link in KNOPPIX
# (C) Klaus Knopper Mar 2004

PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin"
export PATH

RFCOMM_CONF="/etc/bluetooth/rfcomm.conf"

XDIALOG_HIGH_DIALOG_COMPAT=1
export XDIALOG_HIGH_DIALOG_COMPAT

XDIALOG_NO_GMSGS=1
export XDIALOG_NO_GMSGS

[ -r /etc/adriane/dialogrc -a ! -r "$HOME/.dialogrc" ] && export DIALOGRC=/etc/adriane/dialogrc

# Get root
[ "`id -u`" != "0" ] && exec sudo "$0" "$@"

export TEXTDOMAINDIR="/usr/share/locale"
export TEXTDOMAIN=bttool

TMP="/tmp/bttool.$$"

bailout(){
rm -f "$TMP" "$RFCOMM_CONF.tmp"
exit $1
}

DIALOG="dialog"
if [ -n "$DISPLAY" ] && type Xdialog >/dev/null 2>&1; then
 ICON="/usr/share/icons/bttool.xpm"
 [ -r "$ICON" ] && ICON="--icon $ICON" || ICON=""
 DIALOG="Xdialog --fixed-font --cr-wrap --left $ICON"
fi

trap bailout 1 2 3 15

# LANGUAGE etc.
[ -f /etc/sysconfig/knoppix ] && . /etc/sysconfig/knoppix
[ -z "$LANG" ] && export LANG
[ -z "$LANGUAGE" ] && export LANGUAGE
[ -z "$CHARSET" ] && export CHARSET

# Messages
SEARCH=$(gettext search)
SEARCH_DEVICE=$(gettext "Search for new bluetooth device")
PLEASE_SELECT=$(gettext "Please select")
TITLE_BTDEV=$(gettext "Bluetooth Devices")
TITLE_BLUES=$(gettext "Available bluetooth devices")
TITLE_COMMENT=$(gettext "Device name")
MESSAGE_BTDEV=$(gettext "Please enter bluetooth address")
MESSAGE_BTPIN=$(gettext "Please enter bluetooth pin")
BT_NAME=$(gettext "New Bluetooth Device Name")
MESSAGE_BT_NAME=$(gettext "Please enter a description for this bluetooth device")
SELECT_DEVICE=$(gettext "Select device %s")
CHANGE_PIN=$(gettext "Change bluetooth pin for device %s")
CHANGE_DESCRIPTION=$(gettext "Change bluetooth description for device %s")
TEST_DEVICE=$(gettext "Test bluetooth connection to %s")
TEST_RESULT_OK=$(gettext "Bluetooth connection to %s OK.")
TEST_RESULT_ERROR=$(gettext "Error connecting to %s")
DELETE_DEVICE=$(gettext "Remove bluetooth device %s from configuration")


# Read or write rfcomm.conf
# rfcomm_conf read - read all entries from rfcomm.conf, return in arrays
# rfcomm_conf add rfcommX hwaddress comment - add device
# rfcomm_conf delete rfcommX - remove device
rfcomm_conf(){
 local tmpdev tmphw tmpcom
 if [ "$2" = "new" ]; then
  rfcomm_conf read
  local count devno
  for ((devno=0; devno < 16; devno++)); do
   tmpdev="rfcomm$devno"
   for ((count=0; count < RFCOMM_count; count++)); do
    [ "${RFCOMM_name[$count]}" = "rfcomm$devno" ] && continue 2
   done
   break
  done
 fi
 case "$1" in
  add|delete)
       [ -n "$tmpdev" ] || tmpdev="$2"; tmphw="$3"; tmpcom="$4"
       rfcomm release "$tmpdev" >/dev/null 2>&1 ;;
    *) tmpdev=""; tmphw=""; tmpcom="" ;;
 esac
 RFCOMM_name=()
 RFCOMM_device=()
 RFCOMM_channel=()
 RFCOMM_comment=()
 RFCOMM_count=0
 name=""
 device=""
 channel=""
 comment=""
 rm -f "$TMP"
 local line endsection
 while read line; do
  endsection=""
  case "$line" in
   rfcomm[0-9]*) name="${line%% *}"; [ "$name" = "$tmpdev" ] && found="yes";;
   device*)      device="${line#*device }"; device="${device%%;*}";;
   channel*)     channel="${line#*channel \"}"; channel="${channel%%;*}";;
   comment*)     comment="${line#*comment* \"}"; comment="${comment%%\";*}";;
   \}*) RFCOMM_name[$RFCOMM_count]="$name"; RFCOMM_device[$RFCOMM_count]="$device"; RFCOMM_channel[$RFCOMM_count]="$channel"; RFCOMM_comment[$((RFCOMM_count++))]="$comment"; endsection="true" ;;
  esac
  [ -n "$tmpdev" -a -z "$found" ] && echo "$line" >>"$TMP"
  [ -n "$endsection" ] && found=""
 done < "$RFCOMM_CONF"
 if [ -n "$tmpdev" ]; then
  if [ "$1" = "add" ]; then
   echo "
$tmpdev {
 bind yes;
# Bluetooth address of the device
 device $tmphw;
# RFCOMM channel for the connection
 channel 1;
# Description of the connection
 comment \"$tmpcom\";
}" >>"$TMP"
   ADD_RESULT="$tmpdev"
   rfcomm bind "$tmpdev" "$tmphw" >/dev/null 2>&1
  fi
  cat "$TMP" > "$RFCOMM_CONF"
 fi
}

# device_dialog rfcommX: select device or change parameters
device_dialog(){
 while true; do
  local ITEM_SELECT=$(printf "$SELECT_DEVICE" "$1")
  local ITEM_CHANGE_PIN=$(printf "$CHANGE_PIN" "$1")
  local ITEM_CHANGE_DESCRIPTION=$(printf "$CHANGE_DESCRIPTION" "$1")
  local ITEM_TEST=$(printf "$TEST_DEVICE" "$1")
  local ITEM_DELETE=$(printf "$DELETE_DEVICE" "$1")
  local DEVICE_MENU=( p "$ITEM_CHANGE_PIN" c "$ITEM_CHANGE_DESCRIPTION" t "$ITEM_TEST" d "$ITEM_DELETE" )
  rm -f "$TMP"
  $DIALOG --title "$1" --menu "$PLEASE_SELECT" 18 75 9 "${DEVICE_MENU[@]}" 2>"$TMP" || return 0
  local SELECTION
  read SELECTION <"$TMP"
  case "$SELECTION" in
   s) return 0 ;;
   p) change_pin "$1" ;;
   c) change_description "$1" ;;
   t) test_connection "$1" ;;
   d) rfcomm_conf delete "$1"; return 1 ;;
  esac
 done
}

# test_connection rfcommX
test_connection(){
 local RC=0
 gauge "$(printf "$TEST_DEVICE" "$1")" &
 local HW="$(get_address "$1")"
 rfcomm release "$1" >/dev/null 2>&1
 rfcomm bind "$1" "$HW"
 rm -f "$TMP"
 if [ -c /dev/"$1" ]; then
  echo "at" | dd of=/dev/"$1" 2>"$TMP"
  RC="$?"
 else
  echo "No such device: $1" >"$TMP"
  RC=1
 fi
 killgauge
 TEST_ERROR="$(printf "$TEST_RESULT_ERROR" "$1")"
 TEST_OK="$(printf "$TEST_RESULT_OK" "$1")"
 if [ "$RC" != "0" ]; then
  $DIALOG --msgbox "$TEST_ERROR:
$(<$TMP)" 10 75
 else
  $DIALOG --msgbox "$TEST_OK" 10 75
 fi
 rm -f "$TMP"
 return "$RC"
}

# get_address rfcommX - return hw address from rfcomm.conf
get_address(){
 local HW i
 rfcomm_conf read || return 1
 for ((i=0; i < RFCOMM_count; i++)); do
  if [ "${RFCOMM_name[$i]}" = "$1" ]; then
   HW="${RFCOMM_device[$i]}" && break
  fi
 done
 echo "$HW"
 return 0
}

# change_description rfcommX
change_description(){
 local HW i
 for ((i=0; i < RFCOMM_count; i++)); do
  if [ "${RFCOMM_name[$i]}" = "$1" ]; then
   COMMENT="${RFCOMM_comment[$i]}" && break
  fi
 done
 HW=$(get_address "$1")
 ITEM_CHANGE_DESCRIPTION=$(printf "$CHANGE_DESCRIPTION" "$1")
 $DIALOG --title "$TITLE_COMMENT" --inputbox "$ITEM_CHANGE_DESCRIPTION" 15 60 "$COMMENT" 2>"$TMP" || return 1
 read COMMENT <"$TMP"
 rm -f "$TMP"
 [ -n "$COMMENT" ] && rfcomm_conf add "$1" "$HW" "$COMMENT"
 return $?
}

# change_pin rfcommX
# OR
# change_pin bluetooth_address
change_pin(){
 local HW=""
 case "$1" in
  rfcomm[0-9]*) # Need to search for hw address in rfcomm.conf
     rfcomm_conf read || return 1
     for ((i=0; i < RFCOMM_count; i++)); do
      if [ "${RFCOMM_name[$i]}" = "$1" ]; then
       HW="${RFCOMM_device[$i]}" && break
      fi
     done ;;
  *:*:*:*:*:*:*) # Relaxed check.
     HW="$1" ;;
 esac
 [ -n "$HW" ] || return 1
 # Special/"unsupported by debian": /etc/bluetooth/pincodes
 local hw pin relax
 local PINCODES=()
 local PINCODES_index=0
 if [ -r /etc/bluetooth/pincodes ]; then
  while read hw pin relax; do
   if [ "$hw" = "$HW" ]; then
    PIN="$pin"
   else
    PINCODES[$((PINCODES_index++))]="$hw"
    PINCODES[$((PINCODES_index++))]="$pin"
   fi
  done < /etc/bluetooth/pincodes
 fi
 rm -f "$TMP"
 $DIALOG --title "$TITLE_BTPIN" --inputbox "$MESSAGE_BTPIN" 15 60 "$PIN" 2>"$TMP" || return 1
 read PIN < "$TMP"
 rm -f "$TMP"
 PINCODES_index=0
 while [ "$PINCODES_index" -lt "${#PINCODES[@]}" ]; do
  echo "${PINCODES[$((PINCODES_index++))]} ${PINCODES[$((PINCODES_index++))]}"
 done > /etc/bluetooth/pincodes
 [ -n "$PIN" ] && echo "$HW $PIN" >>/etc/bluetooth/pincodes
 local dir
 for dir in /var/lib/bluetooth/*:*; do
  [ -r "$dir"/pincodes ] || ln -snf /etc/bluetooth/pincodes "$dir"/pincodes
 done
 return 0
}

# Display list of devices in /etc/bluetooth/rfcomm.conf and "scan" option.
# SELECT_RESULT contains 
select_rfcomm_device(){
 while true; do
  SELECT_RESULT=""
  rfcomm_conf read
  RFCOMM_LIST=()
  RFCOMM_LIST_count=0
  for ((i=0; i < RFCOMM_count; i++)); do
   RFCOMM_LIST[$((RFCOMM_LIST_count++))]="${RFCOMM_name[$i]}"
   RFCOMM_LIST[$((RFCOMM_LIST_count++))]="${RFCOMM_comment[$i]}"
  done
  RFCOMM_DEV=( "${RFCOMM_LIST[@]}" "$SEARCH" "$SEARCH_DEVICE" )
  $DIALOG --clear --title "$TITLE_BTDEV" --menu "$PLEASE_SELECT" 18 75 9 "${RFCOMM_DEV[@]}" 2>"$TMP" || return 1
  read SELECTION <"$TMP"
  case "$SELECTION" in
   $SEARCH)
    if scan_device && [ -n "${SCAN_RESULT[0]}" ] && rfcomm_conf add new "${SCAN_RESULT[0]}" "${SCAN_RESULT[1]}" && [ -n "$ADD_RESULT" ]; then
     if device_dialog "$ADD_RESULT"; then
      SELECT_RESULT="$ADD_RESULT"
      return 0
     fi
    fi
   ;;
   *)
    if [ -n "$SELECTION" ] && device_dialog "$SELECTION"; then
     SELECT_RESULT="$SELECTION"
     return 0
    fi
   ;;
  esac
 done
}

gauge(){
rm -f "$TMP.done"
status=0
while [ ! -e "$TMP.done" ]; do echo "$status" ; status="`expr \( 100 - $status \) / 4 + $status`"; sleep 2; done | $DIALOG --title "$TITLE_GAUGE" --gauge "$1" 8 75 0
}

# Stop status bar
killgauge(){
touch "$TMP.done" ; wait ; rm -f "$TMP.done"
}

# Returns BT-MAC COMMENT
scan_device(){
 SCAN_RESULT=()
 local count=0
 local HW
 local NAME
 if ! pidof bluetoothd >/dev/null 2>&1 && type -p bluetoothd >/dev/null 2>&1; then
  bluetoothd
 elif ! pidof hcid >/dev/null 2>&1 && type -p hcid >/dev/null 2>&1; then
  hcid
 fi
 gauge "Device-Scan..." &
 while read HW NAME; do
  BLUES[$((count * 2))]="$HW"; BLUES[$((count * 2 + 1))]="$NAME"
  count=$((count + 1))
 done <<EOT
$(hcitool scan 2>/dev/null | awk '/[0-9]:[0-9]/{print $0}')
EOT
 killgauge
 if [ -n "${BLUES[0]}" ]; then
  $DIALOG --title "$TITLE_BLUES" --menu "$MESSAGE_BLUES" 18 75 9 "${BLUES[@]}" 2>"$TMP" || return 1
 else
  $DIALOG --title "$TITLE_BTDEV" --inputbox "$MESSAGE_BTDEV" 15 60 "00:00:00:00:00:00" 2>"$TMP" || return 1
 fi
 read HW <"$TMP"
 [ -n "$HW" ] || return 1
 local i
 for ((i=0; i<count; i++)); do
  if [ "${BLUES[$((i * 2))]}" = "$HW" ]; then
   NAME="${BLUES[$((i * 2 + 1))]}"
   break
  fi
 done
 if [ -z "$NAME" ]; then
  $DIALOG --title "$BT_NAME" --inputbox "$MESSAGE_BT_NAME" 15 60 "$BT_NAME" 2>"$TMP" || return 1
  read NAME <"$TMP"
 fi
 SCAN_RESULT=( "$HW" "$NAME" )
 return 0
}
  
# MAIN
outoutfile=""
#while [ -n "$1" ]; do
# case "$1" in
#  -o) shift; outputfile="$1" ;;
#  *)
#    echo "Usage: $0 [ -o outputfile ]"
#    echo "       configuration of rfcomm devices,"
#    echo "       -o outputfile writes selected rfcomm device to outputfile"
#    ;;
# esac
# shift
#done

select_rfcomm_device || bailout $?
[ -n "$outputfile" ] && echo "$SELECT_RESULT" >"$outputfile"

bailout 0
